#pragma rtGlobals=3		// Use modern global access method.

//Functions listed here are Explicitly for version 2.1 of Ellis.  Each function has a setup function and an executable fit function.
Menu "Ellis2" 
	Submenu "1. Run your setup function"
		"1.1 Setup Randles", setupZrandles()
		"1.2 Setup QPE", setupZqandles()
		"1.3 Setup QPE + Warburg", setupZqandlesW()
		"1.4 Setup Randles + Warburg", setupZrandlesW()
		"1.5 Setup Randles2", setupZrandles2()
		"1.6 Setup Qandles2", setupZqandles2()
		"1.7 Setup Qandles + Porous Bounded Warburg", setupZqandlesB()
		"1.8 Setup Qandles + Bounded Warburg", setupZqandlesT()
		"1.9 Setup Qandles2 parallel", setupZqandles2P()
		"1.11 Setup balanced transmission line", setupZunifiedtlineC()
		"1.X Setup user function", Execute/q "setup"+testsetupvals[0]+"()"
		"n-degree voight", LoadZvN()
	End
End

Function setupZrandles()

	Make/O/D limits ={{0,0,0},{1e3,0.1,1e8}}		// make the lower and upper limits
	Make/O/D coefs = {100,0.000001,3e3}			// make the initial coefficients
	make/T/O coeffnames={"RS","Cdl","RP"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="Zrandles"
	wave/t testsetupvals
	testsetupvals[0]="Zrandles"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function Zrandles(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw//declare wave names.  Don't edit this
	wave/c complexer // don't edit this.  Note that you must have a properly scaled complexer wave in your data directory.
	variable/g midpointt //pull in midpoint value.  This is what divides the real and imaginary parts.
	variable/g/c ki//pull in complex constant
	variable Rs, Rp, Cdl //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Cdl=w[1]
	Rp=w[2]

	// Calculate your function here.  Make sure to keep track of temporary waves and variables.

	complexer[0,midpointt-1] = Rs+1/(1/Rp+1/(ki/-(Cdl*freqw)))  //Compute the final value of the complex function here.
	complexcat[0,midpointt-1]=real(complexer[p]) // don't edit this line.  Writes real part to complexcat first half
	complexcat[midpointt, ]=imag(complexer[p-midpointt]) //don't edit this line.  Writes imaginary part to complexcat second part
End

Function setupZrandlesW()
	Make/O/D limits ={{0,0,0,0},{1e5,0.1,1e6,1000}}		// make the lower and upper limits
	Make/O/D coefs = {5e4,0.0001,5e5,5e4}			// make the initial coefficients
	make/T/O coeffnames={"RS","Cdl","RP","sigmaW"}	// make a list of coefficient names
	string/g currentmodel="ZrandlesW"
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	wave/t testsetupvals
	testsetupvals[0]="ZrandlesW"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function ZrandlesW(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/c complexer
	variable/g midpointt
	variable/g/c ki,kii //pull in complex constants.  Note that ki = i and kii = 1-i
	variable Rs, Rp, Cdl,sigmaW //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Cdl=w[1]
	Rp=w[2]
	sigmaW=w[3]

	complexer[0,midpointt-1] = Rs+1/(1/(Rp+sigmaW*kii/sqrt(freqw))+1/(ki/-(Cdl*freqw)))
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZqandles()
	Make/O/D limits ={{0,0,0,0},{100,0.1,1e6,1}}		// make the lower and upper limits
	Make/O/D coefs = {50,0.00001,5e4,0.8}			// make the initial coefficients
	make/T/O coeffnames={"RS","Qdl","RP","n"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="Zqandles"
	wave/t testsetupvals
	testsetupvals[0]="Zqandles"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End


Threadsafe Function Zqandles(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw 
	wave/c complexer
	variable/g  midpointt
	variable/g/c ki
	variable Rs, Rp, Qdl,n 
	
	Rs=w[0]
	Qdl=w[1]
	Rp=w[2]
	n=w[3]

	complexer[0,midpointt-1]=Rs+1/(1/Rp+(Qdl*(ki*freqw)^n))
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZqandlesW()
	Make/O/D limits ={{0,0,0,0,0},{100,0.1,1e6,1,1000}}		// make the lower and upper limits
	Make/O/D coefs = {50,0.0001,5e5,0.8,100}			// make the initial coefficients
	make/T/O coeffnames={"RS","Qdl","RP","n","sigmaW"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="ZqandlesW"
	wave/t testsetupvals
	testsetupvals[0]="ZqandlesW"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function ZqandlesW(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/d/c complexer //pull in temporary complex wave
	wave/d realout, imagout
	variable/g  midpointt //pull in midpoint
	variable/g/c ki, kii //pull in complex constant
	variable Rs, Rp, Qdl,n,sigmaW //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Qdl=w[1]
	Rp=w[2]
	n=w[3]
	sigmaW=w[4]
	complexer[0,midpointt-1] = Rs+1/(1/(Rp+sigmaW*kii/sqrt(freqw))+(Qdl*(ki*freqw)^n))
	complexcat[0,midpointt-1]=real(complexer)
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

#pragma rtGlobals=3	// Use modern global access method.
c#pragma rtGlobals=3		// Use modern global access method.

Function setupZPDMcomplex()
	Make/O/D limits ={{0.1,0.1,0.1,0.01,0.1,0.1,0.1,0,0,0,0.000000000001,0.000000000001,0.0000000001,0.000000000001,0.000000000001,0.0000000001,-2,1e-8,273.15,0,6e3,1e-6,0.01,1000,6e4,1000,2,0,0.5},{1,1,1,1,1,1,1,2,8,8,1e-8,1e-8,1e-5,1e-8,1e-8,1e-3,2,1,375,14,6e8,1e-4,1,30000,1e5,30000,1000,1e-6,0.9}}		// make the lower and upper limits
	Make/O/D coefs = {0.303321,0.657813,0.307211,0.172236,0.388999,0.183513,0.4056,0.6,3,2.67,7.57E-07,1.58E-06,3.32E-11,1.95E-06,1.21E-06,1.36E-06,-0.2470,-2.03E-05,298,6.3,6.00E+06,1.48E-03,6.55E-01,7746.21,99655.3,7540.03,11.05,0,0.87}			// make the initial coefficients
	make/T/O coeffnames={"a","a1","a2","a3","a4","a5","a7","n","d","chi","k10","k20","k30","k40","k50","k70","V","I","T","pH","epsilon","Qdl","RtO2","WarbO2","RtH2","WarbOx","RS","Kwear","nQ"}	// make a list of coefficient names
	make/O/I heldlist={0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,1,1,0}		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="ZPDMcomplex"
	wave/t testsetupvals
	testsetupvals[0]="ZPDMcomplex"
	make/t/o tempwavenames={"Zpdm","complexer"}
	preflight1()
	//make/O/D R,F,bet,volmol,pot0fs,CH,CH0,gam,wear,c,Lss,a1,a2,a3,a4,a5,a7,b1,b2,b3,c1,c2,c3,c4,c5,c7,k1,k2,k3,k4,k5,k7,theta,Cvm,Cinter,Iv,Ii,IL,Icmv,Icvm,Icil,CMV0,CMI0,CML0,CIVL,CiIL,CiL0,AA,BB,CC,DD
	//make/O/D wang,EE,FF,GG,HH,KK,MM,REALL,IMAGG
	//make/O/c Zpdm,Zcdl,Za,Zc,Zimp
End


Threadsafe Function ZPDMcomplex(w,complexcat,wang) : FitFunc
	Wave w
	wave wang
	wave complexcat
	variable/g midpointt
	Variable/c/g ki, kii
	Variable R,F,bet,volmol,pot0fs,CH,CH0,gam,wear,c,Lss,a1,a2,a3,a4,a5,a7,b1,b2,b3,c1,c2,c3,c4,c5,c7,k1,k2,k3,k4,k5,k7,theta,Cvm,Cinter,Iv,Ii,IL,Icmv,Icvm,Icil,CMV0,CMI0,CML0,CIVL,CiIL,CiL0,AA,BB,CC,DD
	wave/c Zpdm, complexer
	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(frequency) =  COMPLICATED
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ frequency
	//CurveFitDialog/ Coefficients 29
	//CurveFitDialog/ w[0] = alpha
	//CurveFitDialog/ w[1] = alpha1
	//CurveFitDialog/ w[2] = alpha2
	//CurveFitDialog/ w[3] = alpha3
	//CurveFitDialog/ w[4] = alpha4
	//CurveFitDialog/ w[5] = alpha5
	//CurveFitDialog/ w[6] = alpha7
	//CurveFitDialog/ w[7] = n
	//CurveFitDialog/ w[8] = d        
	//CurveFitDialog/ w[9] = chi
	//CurveFitDialog/ w[10] = k1o
	//CurveFitDialog/ w[11] = k2o
	//CurveFitDialog/ w[12] = k3o
	//CurveFitDialog/ w[13] = k4o
	//CurveFitDialog/ w[14] = k5o
	//CurveFitDialog/ w[15] = k7o
	//CurveFitDialog/ w[16] = V
	//CurveFitDialog/ w[17] = I
	//CurveFitDialog/ w[18] = T
	//CurveFitDialog/ w[19] = pH
	//CurveFitDialog/ w[20] = epsilon
	//CurveFitDialog/ w[21] = Cdl
	//CurveFitDialog/ w[22] = RtO2
	//CurveFitDialog/ w[23] = WarbO2
	//CurveFitDialog/ w[24] = RtH2
	//CurveFitDialog/ w[25] = WarbOx	
	//CurveFitDialog/ w[26] = RS 
	//CurveFitDialog/ w[27] = Kwear  
	//CurveFitDialog/ w[28] = nQ  
	
	R=8.31447 // Perfect gas constant
	F=96485.34 // Faraday's constant
	bet=-0.03000 //  effect of pH on the potential across the barrier layer 
	volmol=14.59 // vol molar Cr3+ (cm3/mol)
	pot0fs=-0.1// V
	// T=T0 no variation of kinetic constants according to temperature
	// K(i)o = K(i)00
	
	CH=10^(-w[19])
	CH0=1
	gam=F/(R*w[18])
	// Lss=0.4e-7
			
		a1=w[1]*(1-w[0])*gam* w[9]
		a2=w[2]*(1-w[0])*gam*w[9]
		a3=w[3]*(1-w[0])*gam*w[9]
		a4=w[4]*w[0]*gam*(w[8]-w[9])
		a5=w[5]*w[0]*gam*(w[8]-w[9])	
		a7=w[6]*w[0]*gam*(w[8]-w[9])
	
		b1=-w[9]*w[1]*gam*w[20]
		b2=-w[9]*w[2]*gam*w[20]	
		b3=-w[9]*w[3]*gam*w[20]

		c1=-w[1]*bet*gam*w[9]
		c2=-w[2]*bet*gam*w[9]	
		c3=-w[3]*bet*gam*w[9]
		c4=w[4]*bet*gam*(w[8]-w[9])
		c5=w[5]*bet*gam*(w[8]-w[9])
		c7=w[6]*bet*gam*(w[8]-w[9])	
 	// Lss due to reaction 3 and reaction 7
	//wear term cm/s according to Archard law
	// wear=coeff*normal load*total displacement in 1 s
	// coeff w[25] close to 3.3e-6
	wear=w[27]*127.5*80e-6
	c=((w[15]*exp(w[6]*(w[8]-w[9])*gam*pot0fs))/(w[12]*exp(-w[3]*w[9]*gam*pot0fs)))*exp(a7*(w[16]-w[26]*w[17]))*exp(c7*w[19])*(CH/CH0)^w[7]+(wear/volmol/(w[12]*exp(-w[3]*w[9]*gam*pot0fs)))
	Lss=(1/b3)*log(c)-(a3*(w[16]-w[22]*w[17]))*(1/b3)-(c3*w[19])*(1/b3)

	
	if (Lss<1e-9)
	Lss=0		
	endif
	if (Lss>1e-5)
	Lss=4e-7			
	endif
	// no wear Lss=(-(a3-a7)/b3)*w[16]+((a3-a7)/b3)*w[22]*w[17]+((c7-c3)/b3)*w[19]-(1/b3)*log(w[13]*exp(w[4]*(w[8]-w[9])*gam*pot0fs)/(w[15]*exp(w[6]*(w[8]-w[9])*gam*pot0fs)))+(1/b3)*log((CH/CH0)^w[7])
		
	// Def ki, different code Matthew
	k1=w[10]*exp(-w[1]*w[9]*gam*pot0fs)*exp(a1*(w[16]-(w[26]*w[17])))*exp(b1*Lss)*exp(c1*w[19])
	k2=w[11]*exp(-w[2]*w[9]*gam*pot0fs)*exp(a2*(w[16]-(w[26]*w[17])))*exp(b2*Lss)*exp(c2*w[19])
	k3=w[12]*exp(-w[3]*w[9]*gam*pot0fs)*exp(a3*(w[16]-(w[26]*w[17])))*exp(b3*Lss)*exp(c3*w[19])	
	k4=w[13]*exp(w[4]*(w[8]-w[9])*gam*pot0fs)*exp(a4*(w[16]-(w[26]*w[17])))*exp(c4*w[19])
	k5=w[14]*exp(w[5]*(w[8]-w[9])*gam*pot0fs)*exp(a5*(w[16]-(w[26]*w[17])))*exp(c5*w[19])
	k7=w[15]*exp(w[6]*(w[8]-w[9])*gam*pot0fs)*exp(a7*(w[16]-(w[26]*w[17])))*exp(c7*w[19])

	// Equilibirum concentrations
	theta= (w[8]-w[9])/w[9]
	Cvm=(k4/k1)*theta
	Cinter=(k2/k5)/theta
	
	// Page 16.19 perso notes
	// Def Intensity
	Iv=F*(w[9]*a1*k1*Cvm+w[9]*a2*k2+w[9]*a3*k3+(w[8]-w[9])*a4*k4+(w[8]-w[9])*a5*k5*Cinter+(w[8]-w[9])*a7*k7*CH^w[7]) // deriv de I p/r  V Iv ds les calculs
	Ii=-Iv*w[22]
	IL=F*(w[9]*k1*b1*Cvm+w[9]*b2*k2+w[9]*b3*k3)
	Icmv=F*w[9]*k1
	Icvm=Icmv
	Icil=F*(w[8]-w[9])*k5
	
	//Def coeff concentrations
	CMV0=(k4/k1)*(a4-a1)
	CMI0=(k4/k1)*(a4-a1)*w[22]
	CML0=(k4/k1)*b1
	
	CIVL=(k2/k5)*(a2-a5)
	CiIL=(k2/k5)*(a2-a5)*w[22]
	CiL0=(k2/k5)*b2
		
	// Def AA BB CC DD	
	AA=Iv+Icmv*theta*Cmv0+(Icil/theta)*CIVL
	BB=IL-Icvm*theta*CML0+(Icil/theta)*CiL0
	CC=1+Icvm*theta*CMI0+(Icil/theta)*CiIL+Ii
	DD=(IL-Icvm*theta*CML0-(Icil/theta)*CiL0)
	
	// Def LV LI
	
//	LV=volmol*(a3*k3-a7*k7*CH^(w[7])/(ki*wang-volmol*b3*k3))
//	LI=volmol*(a3*k3*w[26]-a7*k7*CH^(w[7])*w[26])/(ki*wang-volmol*b3*k3)
//	
//	Zpdm=(CC+LI*DD)/(AA+LV*BB)
	
	Zpdm=(CC+(volmol*(a3*k3*w[26]-a7*k7*CH^(w[7])*w[26])/(ki*wang-volmol*b3*k3))*DD)/(AA+(volmol*(a3*k3-a7*k7*CH^(w[7])/(ki*wang-volmol*b3*k3)))*BB)
	 
	// Def electrical equivalent circuit
	// Circuit Rs-(Cbl/Zcath/(Zf+Warb))
	// Zc=RtH2/(RtO2+ZWO2)
	// w[21] = Cdl; w[22] = RtO2; w[23] = WarbO2; w[24] = RtH2; w[25] = WarbOx; w[26] = RS 
	// Warburg impedance in the film + Warburg impedance cathodic reaction

//	Zcpe(dl)=1/(w[21]*(ki*wang)^(nQ))
//	Za=Zpdm+(w[25]/(sqrt(wang)*kii))
//	Zc=w[22]+(w[23]/(sqrt(wang)*kii))
//	Zimp=1/((1/Zcdl)+(1/Za)+(1/Zc))

	complexer[0,midpointt-1]= w[26]+1/(((w[21]*(ki*wang)^(w[28])))+(1/(Zpdm+(w[25]/(sqrt(wang)*kii))))+(1/(w[22]+(w[23]/(sqrt(wang)*kii)))))
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZrandles2()
	Make/O/D limits ={{0,0,0,0,0},{1e5,0.1,1e6,0.1,1e6}}		// make the lower and upper limits
	Make/O/D coefs = {5e4,0.0001,5e5,0.00001,5e5}			// make the initial coefficients
	make/T/O coeffnames={"RS","C1","R1","C2","R2"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="Zrandles2"
	wave/t testsetupvals
	testsetupvals[0]="Zrandles2"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function Zrandles2(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/c complexer
	variable/g midpointt
	variable/g/c ki //pull in complex constant
	variable Rs, R1, C1, R2, C2 //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	C1=w[1]
	R1=w[2]
	C2=w[3]
	R2=w[4]

	complexer[0,midpointt-1]=Rs+1/(1/(R1+1/(1/R2+1/(-ki/(C2*freqw))))+1/(ki/-(C1*freqw)))
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZqandles2()
	Make/O/D limits ={{0,0,0,0,0,0,0},{1e5,0.1,1e6,0.1,1e6,1,1}}		// make the lower and upper limits
	Make/O/D coefs = {5e4,0.0001,5e5,0.00001,5e5,0.8,0.8}			// make the initial coefficients
	make/T/O coeffnames={"RS","Q1","R1","Q2","R2","n1","n2"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="Zqandles2"
	wave/t testsetupvals
	testsetupvals[0]="Zqandles2"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function Zqandles2(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/c complexer
	variable/g midpointt
	variable/g/c ki //pull in complex constant
	variable Rs, R1, Q1, R2, Q2, n1, n2 //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Q1=w[1]
	R1=w[2]
	Q2=w[3]
	R2=w[4]
	n1=w[5]
	n2=w[6]

	complexer[0,midpointt-1]=Rs+1/(1/R1+Q1*(ki*freqw)^n1)+1/(1/R2+Q2*(Ki*freqw)^n2) //Rs-R1||Q1-R2||Q2
//	complexer[0,midpointt-1]=Rs+1/(1/(R1+1/(1/R2+1/(1/(Q2*(ki*freqw)^n2))))+1/(1/(Q1*(ki*freqw)^n2)))
//	complexer[0,midpointt-1]=Rs+1/(1/(Q1*(ki*freqw)^n1)+1/(R1+1/(1/R2+1/(Q2*(ki*freqw)^n2))))
//	complexer[0,midpointt-1]=Rs+1/(1/(R1+1/(1/R2+1/(ki/-(Q1*freqw)^n1)))+1/(ki/-(Q2*freqw)^n2))
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZqandles2P()
	Make/O/D limits ={{0,0,0,0,0,0,0},{1e5,0.1,1e6,0.1,1e6,1,1}}		// make the lower and upper limits
	Make/O/D coefs = {5e4,0.0001,5e5,0.00001,5e5,0.8,0.8}			// make the initial coefficients
	make/T/O coeffnames={"RS","Q1","R1","Q2","R2","n1","n2"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="Zqandles2P"
	wave/t testsetupvals
	testsetupvals[0]="Zqandles2P"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function Zqandles2P(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/c complexer
	variable/g midpointt
	variable/g/c ki //pull in complex constant
	variable Rs, R1, Q1, R2, Q2, n1, n2 //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Q1=w[1]
	R1=w[2]
	Q2=w[3]
	R2=w[4]
	n1=w[5]
	n2=w[6]
	complexer[0,midpointt-1]=Rs+1/(1/(Q1*(ki*freqw)^n1) + 1/(R1 + 1/(1/R2 + 1/(Q2*(ki*freqw)^n2))))
//	complexer[0,midpointt-1]=Rs+1/(1/R1+Q1*(ki*freqw)^n1)+1/(1/R2+Q2*(Ki*freqw)^n2) //Rs-R1||Q1-R2||Q2
//	complexer[0,midpointt-1]=Rs+1/(1/(R1+1/(1/R2+1/(1/(Q2*(ki*freqw)^n2))))+1/(1/(Q1*(ki*freqw)^n2)))
//	complexer[0,midpointt-1]=Rs+1/(1/(Q1*(ki*freqw)^n1)+1/(R1+1/(1/R2+1/(Q2*(ki*freqw)^n2))))
//	complexer[0,midpointt-1]=Rs+1/(1/(R1+1/(1/R2+1/(ki/-(Q1*freqw)^n1)))+1/(ki/-(Q2*freqw)^n2))
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZunifiedtline()
	wave freqdata
	Make/O/D limits ={{0,0,0,0,0,0,0,0,0,0},{1,50000,.01,.01,1,1,1,1,50000,50}}		// make the lower and upper limits
	Make/O/D coefs = {.05,5000,.00001,.00000001,.9,.9,.000001,.05,300,100}			// make the initial coefficients
	make/T/O coeffnames={"r1","r2","qa1","qa2","na1","na2","L","ra","rb","rs"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	make/o/c/n=(numpnts(freqdata)) complexer
	string/g currentmodel="Zunifiedtline"
	wave/t testsetupvals
//	wave/c ksi,za,lam,slam,clam
	testsetupvals[0]="Zunifiedtline"
	make/o/c/d/n=(numpnts(freqdata)) ksi
	make/o/c/d/n=(numpnts(freqdata)) za
	make/o/c/d/n=(numpnts(freqdata)) lam
	make/o/c/d/n=(numpnts(freqdata)) slam
	make/o/c/d/n=(numpnts(freqdata)) clam
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Function setupZunifiedtline0()
	wave freqdata
	Make/O/D limits ={{0,0,0,0,0,0,0,0,0,0,0,0,0},{100,1000,1,1e6,.01,1,1e6,.01,1,10000,.01,1,1}}		// make the lower and upper limits
	Make/O/D coefs = {50,500,.9,1e4,.00001,.9,1e4,.00001,100,.9,.00001,.9,.001}			// make the initial coefficients
	make/T/O coeffnames={"Rs","r1","r2","ra","qa","na","rb","qb","nb","rk","qk","nk","L"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	make/o/c/n=(numpnts(freqdata)) cx
	string/g currentmodel="Zunifiedtline0"
	wave/t testsetupvals
//	wave/c ksi,za,lam,slam,clam
	testsetupvals[0]="Zunifiedtline0"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Function Zunifiedtline0(w,complexcat,u): fitfunc
	wave complexcat, w, u//declare wave names.  Don't edit this
	wave/c cx // don't edit this.  Note that you must have a properly scaled complexer wave in your data directory.
	variable/g midpointt //pull in midpoint value.  This is what divides the real and imaginary parts.
	variable o = midpointt
	variable/g/c ki//pull in complex constant
	variable/c j=ki
	variable a,b,k,c,d,g,h,i,x,y,s,m,L
	
	// assumes that x2<<x1 | x2=0

	s=w[0]//	Rs
	x=w[1]//	r1
	y=w[2]//	r2
	a=w[3]//	ra
	g=w[4]//	qa
	c=w[5]//	na
	b=w[6]//	rb
	h=w[7]//	qb
	d=w[8]//	nb
	k=w[9]//	rk
	i=w[10]//	qk
	m=w[11]//	nk
	L=w[12]//	L


	cx[0,o-1]=s+(a*(b*cosh((L*sqrt(x+i*k*(j*u)^m*x))/sqrt(k))+(1+b*h*(j*u)^d)*x*sqrt(k/(x+i*k*(j*u)^m*x))*sinh(L/sqrt(k/(x+i*k*(j*u)^m*x)))))/((1+a*g*(j*u)^c)*(1+b*h*(j*u)^d)*((a/(1+a*g*(j*u)^c)+b/(1+b*h*(j*u)^d))*cosh((L*sqrt(x+i*k*(j*u)^m*x))/sqrt(k))+((k/(1+i*k*(j*u)^m)+(a*b)/((1+a*g*(j*u)^c)*(1+b*h*(j*u)^d)*x))*sqrt(x+i*k*(j*u)^m*x)*sinh((L*sqrt(x+i*k*(j*u)^m*x))/sqrt(k)))/sqrt(k)))
	complexcat[0,o-1]=real(cx[p]) // don't edit this line.  Writes real part to complexcat first half
	complexcat[o, ]=imag(cx[p-o]) //don't edit this line.  Writes imaginary part to complexcat second part
End

//Function Zunifiedtline(w,complexcat,freqw): fitfunc
//	wave complexcat, w, freqw//declare wave names.  Don't edit this
//	wave/c complexer // don't edit this.  Note that you must have a properly scaled complexer wave in your data directory.
//	variable/g midpointt //pull in midpoint value.  This is what divides the real and imaginary parts.
//	variable/g/c ki//pull in complex constant
//	variable r1,rs,qa1,qa2,na1,na2,L,r2,ra,rb//declare variable names; you can edit this
//	variable x1,x2,zb
//	wave/c ksi,za,lam
//	variable o = midpointt
//	
//	
//	r1=w[0] //assign your variable names to the solution wave.  Edit this.
//	r2=w[1]
//	qa1=w[2]
//	qa2=w[3]
//	na1=w[4]
//	na2=w[5]
//	L=w[6]
//	r3=w[7]
//	Rs=w[8]
//	r4=w[9]
//
//	
//	Za = ra/(1+ra*qa*(ki*freqw)^na
//	Zb = rb/(1+rb*qb*(ki*freqw)^nb
//	Zk = rk/(1+rk*qk*(ki*freqw)^nk
////	lam=sqrt(ksi/x1)
////	lam = sqrt(ksi/(x1 + x2))
////	slam=sinh(L/lam)
////	clam=cosh(L/lam)	
////	complexer[0,midpointt-1]=Rs+(lam*slam*x1^2*Za+clam*x1*Za*Zb)/(x1*(lam*slam*x1+(slam*Za*Zb)/(lam*x1)+clam*(Za+Zb))) //such that x2=r2=0
//	complexer[0,midpointt-1]=Rs+(sqrt(ksi/x1)*sinh(L/sqrt(ksi/x1))*x1^2*Za+cosh(L/sqrt(ksi/x1))*x1*Za*Zb)/(x1*(sqrt(ksi/x1)*sinh(L/sqrt(ksi/x1))*x1+(sinh(L/sqrt(ksi/x1))*Za*Zb)/(sqrt(ksi/x1)*x1)+cosh(L/sqrt(ksi/x1))*(Za+Zb)))
////	complexer[0,midpointt-1]=Rs+(lam*sinh(L/lam)*x1^2*Za+cosh(L/lam)*x1*Za*Zb)/(x1*(lam*sinh(L/lam)*x1+(sinh(L/lam)*Za*Zb)/(lam*x1)+cosh(L/lam)*(Za+Zb)))
////	complexer[0,midpointt-1]=Rs+(L*x1*x2*(lam^2*slam*(x1+x2)^2+slam*Za*Zb+clam*lam*(x1+x2)*(Za+Zb))+lam*((2*x1*x2+clam*(x1^2+x2^2))*Za*Zb+lam*slam*(x1+x2)*(x1^2*Za+x2^2*Zb)))/((x1+x2)*(lam^2*Slam*(x1+x2)^2+slam*Za*Zb+clam*lam*(x1+x2)*(Za+Zb)))
////	complexer[0,midpointt-1]=rs+(L*lam*x1*x2*(x1+x2)*sinh(L/lam)+x1*(lam*x1*sinh(L/lam)+L*x2*cosh(L/lam))*za+x2*(lam*x2*sinh(L/lam)+L*x1*cosh(L/lam))*zb+(2*x1*x2+(x1^2+x2^2)*cosh(L/lam)+L*x1*x2*sinh(L/lam)/lam)*za*zb/(x1+x2))/(x1+x2)/(lam*(x1+x2)*sinh(L/lam)+(za+zb)*cosh(L/lam)+Za*Zb*sinh(L/lam)/(lam*(x1+x2)))
////	complexer[0,midpointt-1]=(L*lam*x1*x2*(x1+x2)*slam+x1*(lam*x1*slam+L*x2*clam)*za+x2*(lam*x2*slam+L*x1*clam)*zb+(2*x1*x2+(x1^2+x2^2)*clam+L*x1*x2*slam/lam)*za*zb/(x1+x2))/(x1+x2)/(lam*(x1+x2)*slam+(za+zb)*clam+Za*Zb*slam/(lam*(x1+x2)))
////	complexer[0,midpointt-1] =1/(x1 + x2)/(lam*(x1 + x2)*sinh(L/lam) + (za + zb)*cosh(L/lam) + za*zb*sinh(L/lam)/(sinh(L/lam)*(x1 + x2)))*(L*lam*x1*x2*(x1 + x2)*sinh(L/lam) + x1*(lam*x1*sinh(L/lam) + L*x2*cosh(L/lam))*za + (2*x1*x2 + (x1^2 + x2^2)*cosh(L/lam) + L/lam*x1*x2*sinh(L/lam))*za*zb/(x1 + x2))
//	complexcat[0,midpointt-1]=real(complexer[p]) // don't edit this line.  Writes real part to complexcat first half
//	complexcat[midpointt, ]=imag(complexer[p-midpointt]) //don't edit this line.  Writes imaginary part to complexcat second part
//End

Function setupZqandlesB()
	Make/O/D limits ={{0,0,0,0,0,0},{100,0.1,1e6,1,1,1e7}}		// make the lower and upper limits
	Make/O/D coefs = {50,0.00001,1e3,0.8,1e-4,1e6}			// make the initial coefficients
	make/T/O coeffnames={"RS","Qdl","RP","n","B","Y0"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="ZqandlesB"
	wave/t testsetupvals
	testsetupvals[0]="ZqandlesB"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function ZqandlesB(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/d/c complexer //pull in temporary complex wave
	wave/d realout, imagout
	variable/g  midpointt //pull in midpoint
	variable/g/c ki//pull in complex constant
	variable Rs, Rp, Qdl,n,B,Y0 //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Qdl=w[1]
	Rp=w[2]
	n=w[3]
	B=w[4]
	Y0=w[5]
	complexer[0,midpointt-1] = Rs+1/(1/(Rp+1/Y0/sqrt(ki*freqw)*tanh(B*sqrt(ki*freqw)))+(Qdl*(ki*freqw)^n))
	complexcat[0,midpointt-1]=real(complexer)
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZqandlesT()
	Make/O/D limits ={{0,0,0,0,0,0},{100,0.1,1e6,1,1,1e7}}		// make the lower and upper limits
	Make/O/D coefs = {50,0.00001,1e3,0.8,1e-4,1e6}			// make the initial coefficients
	make/T/O coeffnames={"RS","Qdl","RP","n","B","Y0"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="ZqandlesT"
	wave/t testsetupvals
	testsetupvals[0]="ZqandlesT"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function ZqandlesT(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/d/c complexer //pull in temporary complex wave
	wave/d realout, imagout
	variable/g  midpointt //pull in midpoint
	variable/g/c ki//pull in complex constant
	variable Rs, Rp, Qdl,n,B,Y0 //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Qdl=w[1]
	Rp=w[2]
	n=w[3]
	B=w[4]
	Y0=w[5]
	complexer[0,midpointt-1] = Rs+1/(1/(Rp+1/Y0/sqrt(ki*freqw)*coth(B*sqrt(ki*freqw)))+(Qdl*(ki*freqw)^n))
	complexcat[0,midpointt-1]=real(complexer)
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZqandlesG()
	Make/O/D limits ={{0,0,0,0,0,0},{100,0.1,1e6,1,1,1e7}}		// make the lower and upper limits
	Make/O/D coefs = {50,0.00001,1e3,0.8,1e-4,1e6}			// make the initial coefficients
	make/T/O coeffnames={"RS","Qdl","RP","n","k","Y0"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="ZqandlesG"
	wave/t testsetupvals
	testsetupvals[0]="ZrandlesG"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function ZqandlesG(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/d/c complexer //pull in temporary complex wave
	wave/d realout, imagout
	variable/g  midpointt //pull in midpoint
	variable/g/c ki//pull in complex constant
	variable Rs, Rp, Qdl,n,k0,Y0 //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Qdl=w[1]
	Rp=w[2]
	n=w[3]
	k0=w[4]
	Y0=w[5]
	complexer[0,midpointt-1] = Rs+1/(1/(Rp+1/Y0/sqrt(k0+ki*freqw))+(Qdl*(ki*freqw)^n))
	complexcat[0,midpointt-1]=real(complexer)
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZunifiedtlineC()
	Make/O/D limits ={{0,0,0,0,0,0,0,0,0,0,0,0,0},{100,1000,1,1e6,.01,1,1e6,.01,1,10000,.01,1,1}}		// make the lower and upper limits
	Make/O/D coefs = {50,500,.9,1e4,.00001,.9,1e4,.00001,100,.9,.00001,.9,.001}			// make the initial coefficients
	make/T/O coeffnames={"Rs","r1","r2","ra","qa","na","rb","qb","nb","rk","qk","nk","L"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="ZunifiedtlineC"
	wave/t testsetupvals
	testsetupvals[0]="ZunifiedtlineC"
	make/t/o tempwavenames={"complexer","lam","slam","clam","Za","Zb","ksi"}
	preflight1()
End

Threadsafe Function ZunifiedtlineC(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/c complexer, lam, slam, clam, Za, Zb, ksi
	variable/g midpointt
	variable/g/c ki //pull in complex constant
	variable Rs,x1,x2,ra,qa,na,rb,qb,nb,rk,qk,nk,L //declare variable names; you can edit this
	
	Rs=w[0]//	Rs
	x1=w[1]//	r1
	x2=w[2]//	r2
	ra=w[3]//	ra
	qa=w[4]//	qa
	na=w[5]//	na
	rb=w[6]//	rb
	qb=w[7]//	qb
	nb=w[8]//	nb
	rk=w[9]//	rk
	qk=w[10]//	qk
	nk=w[11]//	nk
	L=w[12]//	L

	ksi = Rk/(1 + Rk*Qk*(ki*freqw)^nk);
	lam = Sqrt(ksi/(x1 + x2));
	slam = Sinh(L/lam);
	clam = Cosh(L/lam);
	Za = Ra/(1 + Ra*Qa*(ki*freqw)^na);
	Zb = Rb/(1 + Rb*Qb*(ki*freqw)^nb);
	

	complexer[0,midpointt-1]=Rs+ 1/(x1 + x2)*(lam*(x1 + x2)*slam + (Za + Zb)*clam + 1/(lam*(x1 + x2))*Za*Zb*slam)^-1*(L*lam*x1*x2*(x1 + x2)*slam + x1*(lam*x1*slam + L*x2*clam)*Za + x2*(lam*x2*slam + L*x1*clam)*Zb + 1/(x1 + x2)*(2*x1*x2 + (x1^2 + x2^2)*clam + L/lam*x1*x2*slam)*Za*Zb)
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
//	complexcat = Numtype(complexcat[p]) == 0 ? complexcat[p] : complexcat[p-1] //check for normalcy
End

Function LoadZvN()
	variable ninput
	Prompt ninput, "n:"		// Set prompt for x param
	DoPrompt "How many Voight units?",ninput
	SetupZvN(ninput)
End

Function setupZvN(n)
	variable n // Number of Voight Elements
	variable/g nV = n //global storage of n

	wave limits 
	//make the lower and upper limits.  This assumes that every voight element is varied through the same range
	string limlow="{0"
	variable ii
	
	for(ii = 0 ; ii <n ; ii += 1)
		limlow=limlow+",0,0"
	endfor
	
	string limhigh="{100"
	for(ii = 0 ; ii <n ; ii += 1)
		limhigh=limhigh+",0.1,1e4"
	endfor
	Execute "Make/O/D limits ={"+limlow+"},"+limhigh+"}}"
	//make the initial coefficients
	wave coefs
	string coe="100"
	for(ii = 0 ; ii <n ; ii += 1)
		coe=coe+",0.000001,3e3"
	endfor
	Execute "Make/O/D coefs ={"+coe+"}"

	//Name the coefficients
	wave/t coeffnames
	make/o/t/n=(2*n+1) coeffnames
	coeffnames[0]="R0"
	for(ii = 0 ; ii <n ; ii += 1)
		coeffnames[2*ii+1]="C"+num2istr(ii+1)
		coeffnames[2*ii+2]="R"+num2istr(ii+1)
	endfor
	//make the heldlist
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="ZvN"

	wave/t testsetupvals
	testsetupvals[0]="ZvN"//load the name of the model into the setup pane
	make/t/o tempwavenames={"complexer"} // only required temporary wave is complexer
	preflight1()
End

Threadsafe Function ZvN(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw//declare wave names.  Don't edit this
	variable/g nV
//	variable coefindex = numpnts(w)
	wave/c complexer // don't edit this.  Note that you must have a properly scaled complexer wave in your data directory.
	variable/g midpointt //pull in midpoint value.  This is what divides the real and imaginary parts.
	variable/g/c ki//pull in complex constant

	complexer[0,midpointt-1]=w[0]
	variable ii
	for(ii = 0 ; ii < nV; ii += 1)
		complexer[0,midpointt-1] += 1/(1/w[2*ii+2]+1/(ki/-(w[2*ii+1]*freqw)))
	endfor
	complexcat[0,midpointt-1]=real(complexer[p]) // don't edit this line.  Writes real part to complexcat first half
	complexcat[midpointt, ]=imag(complexer[p-midpointt]) //don't edit this line.  Writes imaginary part to complexcat second part
End
